==> toycars-0.3.0.tar.gz: toycars-0.3.0/src/PacejkaCurve.cpp <==
/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 20/01/05.
 */

#include <cstdlib>
#include "PacejkaCurve.h"

PacejkaCurve::PacejkaCurve(double inD, double inC, double inB, double inE, unsigned short inSamples, double domainMax)
	: granularity(inSamples/domainMax), samples(inSamples), D(inD), C(inC), B(inB), E(inE)
{
	// Allocate memory for table of samples
	
	data = new double [samples];
	if (data == NULL)
		throw 1;	// nb: perhaps do some better error handling here?
	
	// Compute peak value of pacejka function using
	// the bisection method on a numerical approximation
	// to the derivate.
	
	const double dx = 1e-6;
	double x1, x2, x, dy;
	
	x1 = 0;
	x2 = 10;
	do
	{
		x = 0.5 * (x1 + x2);
		dy = pacejkaFn(x + dx) - pacejkaFn(x);
		(dy < 0) ? (x2 = x) : (x1 = x);
	} while (fabs(dy) > dx*dx);
	
	peak = x;
	
	// calculate function limit at infinity
	
	if (E == -1)
		limit = D*sin(C*atan(M_PI*0.5));
	else if (E > -1)
		limit = D*sin(C*M_PI*0.5);
	else if (E < -1)
		limit = -D*sin(C*M_PI*0.5);
	
	// calculate data and fill table
	
	for (int i = 0; i < samples; i++)
	{
		data[i] = pacejkaFn(i / granularity);
	}
}

PacejkaCurve::~PacejkaCurve()
{
	delete [] data;
}

double PacejkaCurve::pacejkaFn(double value)
{
	value *= B;
	return D*sin(C*atan(value - E*(value - atan(value))));
}

// Pacejka magic formula
// D*sin(C*atan(B*theta - E*(B*theta - atan(B*theta))))
// slope at origin == B*C*D
// limit at infinity ==	D*sin(C*pi/2) if E > -1
//						D*sin(C*atan(pi/2)) if E == -1
//						-D*sin(C*pi/2) if E < -1

==> toycars-0.3.3.tar.gz: toycars-0.3.3/src/PacejkaCurve.cpp <==
/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Henner Zilibowitz on 20/01/05.
 */

#include <cstdlib>
#include "PacejkaCurve.h"

PacejkaCurve::PacejkaCurve(double inD, double inC, double inB, double inE, unsigned short inSamples, double domainMax)
	: granularity(inSamples/domainMax), samples(inSamples), D(inD), C(inC), B(inB), E(inE)
{
	// Allocate memory for table of samples
	
	data = new double [samples];
	if (data == NULL)
		throw 1;	// nb: perhaps do some better error handling here?
	
	// Compute peak value of pacejka function using
	// the bisection method on a numerical approximation
	// to the derivate.
	
	const double dx = 1e-6;
	double x1, x2, x, dy;
	
	x1 = 0;
	x2 = 10;
	do
	{
		x = 0.5 * (x1 + x2);
		dy = pacejkaFn(x + dx) - pacejkaFn(x);
		(dy < 0) ? (x2 = x) : (x1 = x);
	} while (fabs(dy) > dx*dx);
	
	peak = x;
	
	// calculate function limit at infinity
	
	if (E == -1)
		limit = D*sin(C*atan(M_PI*0.5));
	else if (E > -1)
		limit = D*sin(C*M_PI*0.5);
	else if (E < -1)
		limit = -D*sin(C*M_PI*0.5);
	
	// calculate data and fill table
	
	for (int i = 0; i < samples; i++)
	{
		data[i] = pacejkaFn(i / granularity);
	}
}

PacejkaCurve::~PacejkaCurve()
{
	delete [] data;
}

double PacejkaCurve::pacejkaFn(double value)
{
	value *= B;
	return D*sin(C*atan(value - E*(value - atan(value))));
}

// Pacejka magic formula
// D*sin(C*atan(B*theta - E*(B*theta - atan(B*theta))))
// slope at origin == B*C*D
// limit at infinity ==	D*sin(C*pi/2) if E > -1
//						D*sin(C*atan(pi/2)) if E == -1
//						-D*sin(C*pi/2) if E < -1
